# author: Chantal Peeters
# date: 02.01.2013

### Computations to illustrate the comparison between the saddlepoint approximation
### and the Monte Carlo Methods ISS and ISSEM in the calculation of sensitivities of
### random sums.

### Random sum C = Compound Geometric Sum with Gamma distributed summands
### We calculate the sensitivity of P(C>x) with respect to the parameter of the geometric
### distribution (p).

### Comparison A: p = 0.5, x variable

source('SimulationGamma_Masswechsel.R');
source('SimulationGamma.R');
source('ApproximationGamma2.R');
source('ExaktGamma.R');
source('ExaktGammaZ.R');
  
### Comparison A: p = 0.5, alpha = 2, beta = 1
# parameter of the Geometric distribution: p = 0.5
# parameters of the Gamma distribution: alpha = 2, beta = 1
p = 0.5;
alpha = 2;
beta = 1;
n = 100000;
genauigkeit = 10^(-14);

x = seq(0.15,40,by = 0.15);

# Compute the exact probability P(C>x) and the sensitivity of P(C>x) with
#   - the Monte Carlo Method ISS (Importance Sampling for Sensitivities)
#   - the Monte Carlo Method ISSEM (ISS with an exponential change of measure)
#   - a nearly exact calculation
#   - the saddlepoint approximation
sensitivity_sim = rep(0,length(x));
sensitivity_sim_M = rep(0,length(x));
sensitivity = rep(0,length(x));
sensitivity_appr = rep(0,length(x));
p_Z = rep(0,length(x))


for (i in 201:length(x)){
	print(i);
	sensitivity_sim[i] = SimulationGamma(x[i],alpha,beta,p,n=n);
	sensitivity_sim_M[i] = SimulationGamma_Masswechsel(x[i],alpha,beta,p,n=n,genauigkeit);
	sensitivity[i] = ExaktGamma(x[i],alpha,beta,p,genauigkeit);
	sensitivity_appr[i] = ApproximationGamma2(x[i],alpha,beta,p,genauigkeit);
	p_Z[i] = ExaktGammaZ(x[i],alpha,beta,p,10^(-17));

}


# values for upper axis
p_r = round(p_Z, digits = 2);
p_r_3 = round(p_Z, digits = 3);
p_s = signif(p_Z, digits = 2);
p_axis_x =c(x[1],x[9],x[16],x[25],x[41],x[93],x[145],x[198],x[250]);
p_axis_p = c(p_r[1],p_r[9],p_r[16],p_r[25],p_r[41],p_r[93],p_r_3[145],p_s[198],p_s[250]);

# absolute and relative errors
absoluter_fehler_sim = sensitivity_sim - sensitivity;
absoluter_fehler_sim_M = sensitivity_sim_M - sensitivity;
absoluter_fehler_appr = sensitivity_appr - sensitivity;

relativer_fehler_sim = absoluter_fehler_sim/sensitivity;
relativer_fehler_sim_M = absoluter_fehler_sim_M/sensitivity;
relativer_fehler_appr = absoluter_fehler_appr / sensitivity;


## Graphs

## Results: sensitivity & absolute error & relative error
# pdf file
pdf(file="A_Masswechsel.pdf",width=6, height=11.7);
par(mfrow=c(3,1),mar=c(5.7,4.1,6.1,2.1));

plot(x,sensitivity_sim_M,type="l",cex.axis=1.4,cex.lab=1.6,ylab="Sensitivitt",mgp=c(2.7,0.7,0),col="blue",lwd=0.5);
lines(x,sensitivity_appr,type="l",lty=2,col="blue",lwd=0.5);
lines(x,sensitivity,type="l",lty=3,col="blue",lwd=0.5);
legend("bottomright", c("Monte Carlo Methode ISSEM","Sattelpunkt Approximation","exakte Berechnung"), cex=1.2,  lty=c(1,2,3),col="blue",lwd=0.5, bty="n");

axis(3,at=p_axis_x,labels=p_axis_p,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Sensitivitt",font=2);


plot(x,absoluter_fehler_sim_M,type="l",cex.axis=1.4,cex.lab=1.6,ylab="absoluter Fehler",mgp=c(2.7,0.7,0),col="blue",lwd=0.5,ylim=c(-0.02,0.02));
lines(x,absoluter_fehler_appr,type="l",lty=2,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
legend("topright", c("Monte Carlo Methode ISSEM","Sattelpunkt Approximation"), cex=1.2,  lty=1:2,col="blue",lwd=0.5, bty="n");

axis(3,at=p_axis_x,labels=p_axis_p,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Absoluter Fehler der Sensitivitt",font=2);


plot(x,relativer_fehler_sim_M,type="l",cex.axis=1.4,cex.lab=1.6,ylab="relativer Fehler",mgp=c(2.7,0.7,0),col="blue",lwd=0.5,ylim=c(-0.03,0.03));
lines(x,relativer_fehler_appr,type="l",lty=2,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
legend("topleft", c("Monte Carlo Methode ISSEM","Sattelpunkt Approximation"), cex=1.2,  lty=1:2,col="blue",lwd=0.5, bty="n");

axis(3,at=p_axis_x,labels=p_axis_p,cex.axis=1.4,mgp=c(1.7,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Relativer Fehler der Sensitivitt",font=2);
dev.off()

# eps file
postscript(file="A_Masswechsel.eps", onefile=FALSE, horizontal=FALSE, width=6, height = 11.7)
par(mfrow=c(3,1),mar=c(5.7,4.1,6.1,2.1));

plot(x,sensitivity_sim_M,type="l",cex.axis=1.4,cex.lab=1.6,ylab="Sensitivitt",mgp=c(2.7,0.7,0),col="blue",lwd=0.5);
lines(x,sensitivity_appr,type="l",lty=2,col="blue",lwd=0.5);
lines(x,sensitivity,type="l",lty=3,col="blue",lwd=0.5);
legend("bottomright", c("Monte Carlo Methode ISSEM","Sattelpunkt Approximation","exakte Berechnung"), cex=1.2,  lty=c(1,2,3),col="blue",lwd=0.5, bty="n");

axis(3,at=p_axis_x,labels=p_axis_p,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Sensitivitt",font=2);


plot(x,absoluter_fehler_sim_M,type="l",cex.axis=1.4,cex.lab=1.6,ylab="absoluter Fehler",mgp=c(2.7,0.7,0),col="blue",lwd=0.5,ylim=c(-0.02,0.02));
lines(x,absoluter_fehler_appr,type="l",lty=2,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
legend("topright", c("Monte Carlo Methode ISSEM","Sattelpunkt Approximation"), cex=1.2,  lty=1:2,col="blue",lwd=0.5, bty="n");

axis(3,at=p_axis_x,labels=p_axis_p,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Absoluter Fehler der Sensitivitt",font=2);


plot(x,relativer_fehler_sim_M,type="l",cex.axis=1.4,cex.lab=1.6,ylab="relativer Fehler",mgp=c(2.7,0.7,0),col="blue",lwd=0.5,ylim=c(-0.03,0.03));
lines(x,relativer_fehler_appr,type="l",lty=2,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
legend("topleft", c("Monte Carlo Methode ISSEM","Sattelpunkt Approximation"), cex=1.2,  lty=1:2,col="blue",lwd=0.5, bty="n");

axis(3,at=p_axis_x,labels=p_axis_p,cex.axis=1.4,mgp=c(1.7,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Relativer Fehler der Sensitivitt",font=2);
dev.off()



## Comparison ISS with ISSEM
par(mfrow=c(1,1),mar=c(5.7,4.1,6.1,2.1));
plot(x[1:200],relativer_fehler_sim[1:200],type="l",lty=2,cex.axis=1.2,cex.lab=1.3,ylab="relativer Fehler",mgp=c(2.5,0.5,0),col="blue",lwd=0.5);
lines(x[1:200],relativer_fehler_sim_M[1:200],type="l",lty=1,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
axis(3,at=p_axis_x[1:8],labels=p_axis_p[1:8],cex.axis=1.2,mgp=c(1.5,0.5,0));
mtext(side=3,line=1.6,cex=1.3,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.2, cex=1.8,"Relativer Fehler der Sensitivitt",font=2);
legend("topleft", c("ohne exponentiellen Masswechsel","mit exponentiellen Masswechsel"), cex=1.2,  lty=2:1,col="blue",lwd=0.5, bty="n");
dev.copy2pdf(file="A_Vergleich_mit_ohne_Masswechsel.pdf",width=8,height=5)
dev.copy2eps(file="A_Vergleich_mit_ohne_Masswechsel.eps",width=8,height=5)
dev.off()
