# author: Chantal Peeters
# date: 02.01.2013

### Computations to illustrate the comparison between the saddlepoint approximation
### and the Monte Carlo Methods ISS and ISSEM in the calculation of sensitivities of
### random sums.

### Random sum C = Compound Geometric Sum with Gamma distributed summands
### We calculate the sensitivity of P(C>x) with respect to the parameter of the 
### geometric distribution (p).

### Comparison B: p = variable, x = 14

source('SimulationGamma.R');
source('SimulationGamma_Masswechsel.R');
source('ApproximationGamma2.R');
source('ExaktGamma.R');
source('ExaktGammaZ.R');
 
### Comparison B: x = 14
# parameters of the Gamma distribution: alpha = 2, beta = 1
x = 14;
alpha = 2;
beta = 1;
n = 100000;
genauigkeit = 10^(-12);
genauigkeit_Z = 10^(-16);

p = seq(0.01,0.99,by=0.005)

# Compute the exact probability P(C>x) and the sensitivity of P(C>x) with
#   - the Monte Carlo Method ISS (Importance Sampling for Sensitivities)
#   - the Monte Carlo Method ISSEM (ISS with an exponential change of measure)
#   - a nearly exact calculation
#   - the saddlepoint approximation

sensitivity_sim = rep(0,length(p))
sensitivity_sim_M = rep(0,length(p))
sensitivity = rep(0,length(p))
sensitivity_appr = rep(0,length(p))
p_Z = rep(0,length(p))

for (i in 1:length(p)){
print(i);
	sensitivity_sim[i] = SimulationGamma(x,alpha,beta,p[i],n=n);
	sensitivity_sim_M[i] = SimulationGamma_Masswechsel(x,alpha,beta,p[i],n=n,genauigkeit);
	sensitivity[i] = ExaktGamma(x,alpha,beta,p[i],genauigkeit_Z);
	sensitivity_appr[i] = ApproximationGamma2(x,alpha,beta,p[i],genauigkeit);
	p_Z[i] = ExaktGammaZ(x,alpha,beta,p[i],genauigkeit_Z);
}

# values for upper axis
p_r = round(p_Z, digits = 2);
p_s = signif(p_Z, digits = 2);
p_axis_p = c(p[2],p[5],p[8],p[12],p[16],p[22],p[29],
p[39],p[53],p[99],p[135],p[162],p[181],p[192]);
p_axis_p_Z = c(p_r[2],p_r[5],p_r[8],p_r[12],p_r[16],p_r[22],p_r[29]
,p_r[38],p_r[53],p_r[99],p_s[135],p_s[162],p_s[181],p_s[192]);

# absolute and relative errors
absoluter_fehler_sim = sensitivity_sim - sensitivity;
absoluter_fehler_sim_M = sensitivity_sim_M - sensitivity;
absoluter_fehler_appr = sensitivity_appr - sensitivity;

relativer_fehler_sim = absoluter_fehler_sim/sensitivity;
relativer_fehler_sim_M = absoluter_fehler_sim_M/sensitivity;
relativer_fehler_appr = absoluter_fehler_appr / sensitivity;



## Graphs

## Results: sensitivity & absolute error & relative error

# pdf file

pdf(file="B_Masswechsel_AF.pdf",width=6, height=11.7);
par(mfrow=c(3,1),mar=c(5.7,4.1,6.1,2.1));


plot(p,sensitivity_sim_M,type="l",cex.axis=1.4,cex.lab=1.6,ylab="Sensitivitt",mgp=c(2.7,0.7,0),col="blue",lwd=0.5);
lines(p,sensitivity_appr,type="l",lty=2,col="blue",lwd=0.5);
lines(p,sensitivity,type="l",lty=3,col="blue",lwd=0.5);
legend("bottomright", c("Monte Carlo Methode ISSEM","Sattelpunkt Approximation","exakte Berechnung"), cex=1.2,  lty=c(1,2,3),col="blue",lwd=0.5, bty="n");

axis(3,at=,p_axis_p,p_axis_p_Z,labels=,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Sensitivitt",font=2);


plot(p,absoluter_fehler_sim_M,type="l",cex.axis=1.4,cex.lab=1.6,ylab="absoluter Fehler",mgp=c(2.7,0.7,0),col="blue",lwd=0.5,ylim=c(-0.15,0.15));
lines(p,absoluter_fehler_appr,type="l",lty=2,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
legend("topright", c("Monte Carlo Methode ISSEM","Sattelpunkt Approximation"), cex=1.2,  lty=1:2,col="blue",lwd=0.5, bty="n");

axis(3,at=,p_axis_p,p_axis_p_Z,labels=,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Absoluter Fehler der Sensitivitt",font=2);


plot(p,relativer_fehler_sim_M,type="l",cex.axis=1.4,cex.lab=1.6,ylab="relativer Fehler",mgp=c(2.7,0.7,0),col="blue",lwd=0.5,ylim=c(-0.04,relativer_fehler_appr[197]));
lines(p,relativer_fehler_appr,type="l",lty=2,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
legend("topleft", c("Monte Carlo Methode ISSEM","Sattelpunkt Approximation"), cex=1.2,  lty=1:2,col="blue",lwd=0.5, bty="n");

axis(3,at=,p_axis_p,p_axis_p_Z,labels=,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Relativer Fehler der Sensitivitt",font=2);
dev.off();



# eps file
postscript(file="B_Masswechsel_AF.eps", onefile=FALSE, horizontal=FALSE, width=6, height = 11)
par(mfrow=c(3,1),mar=c(5.7,4.1,6.1,2.1));

plot(p,sensitivity_sim_M,type="l",cex.axis=1.4,cex.lab=1.6,ylab="Sensitivitt",mgp=c(2.7,0.7,0),col="blue",lwd=0.5);
lines(p,sensitivity_appr,type="l",lty=2,col="blue",lwd=0.5);
lines(p,sensitivity,type="l",lty=3,col="blue",lwd=0.5);
legend("bottomright", c("Monte Carlo Methode ISSEM","Sattelpunkt Approximation","exakte Berechnung"), cex=1.2,  lty=c(1,2,3),col="blue",lwd=0.5, bty="n");

axis(3,at=,p_axis_p,p_axis_p_Z,labels=,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Sensitivitt",font=2);


plot(p,absoluter_fehler_sim_M,type="l",cex.axis=1.4,cex.lab=1.6,ylab="absoluter Fehler",mgp=c(2.7,0.7,0),col="blue",lwd=0.5,ylim=c(-0.15,0.15));
lines(p,absoluter_fehler_appr,type="l",lty=2,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
legend("topright", c("Monte Carlo Methode ISSEM","Sattelpunkt Approximation"), cex=1.2,  lty=1:2,col="blue",lwd=0.5, bty="n");

axis(3,at=,p_axis_p,p_axis_p_Z,labels=,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Absoluter Fehler der Sensitivitt",font=2);


plot(p,relativer_fehler_sim_M,type="l",cex.axis=1.4,cex.lab=1.6,ylab="relativer Fehler",mgp=c(2.7,0.7,0),col="blue",lwd=0.5,ylim=c(-0.04,relativer_fehler_appr[197]));
lines(p,relativer_fehler_appr,type="l",lty=2,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
legend("topleft", c("Monte Carlo Methode ISSEM","Sattelpunkt Approximation"), cex=1.2,  lty=1:2,col="blue",lwd=0.5, bty="n");

axis(3,at=,p_axis_p,p_axis_p_Z,labels=,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Relativer Fehler der Sensitivitt",font=2);
dev.off();


## Comparison ISS with ISSEM
par(mfrow=c(1,1),mar=c(5.7,4.1,6.1,2.1));
plot(p,relativer_fehler_sim,type="l",lty=2,cex.axis=1.2,cex.lab=1.2,ylab="relativer Fehler",mgp=c(2.5,0.5,0),col="blue",lwd=0.5);
lines(p,relativer_fehler_sim_M,type="l",lty=1,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);

axis(3,at=p_axis_p,labels=p_axis_p_Z,cex.axis=1.2,mgp=c(1.5,0.5,0));
mtext(side=3,line=1.6,cex=1.3,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.2, cex=1.8,"Relativer Fehler der Sensitivitt",font=2);
legend("topleft", c("ohne exponentiellen Masswechsel","mit exponentiellen Masswechsel"), cex=1.2,  lty=2:1,col="blue",lwd=0.5, bty="n");
dev.copy2pdf(file="B_Vergleich_mit_ohne_Masswechsel_AF.pdf",width=8,height=5)
dev.copy2eps(file="B_Vergleich_mit_ohne_Masswechsel_AF.eps",width=8,height=5)
dev.off()
