# author: Chantal Peeters
# date: 02.01.2013

### Computations to illustrate the comparison between the saddlepoint approximation
### and the Monte Carlo Methods ISS and ISSEM in the calculation of sensitivities of
### random sums.

### Random sum C = Compound Poisson Sum with Gamma distributed summands
### We calculate the sensitivity of P(C>x) with respect to parameter of the Poisson
### distribution (lambda).

### Comparison A: lambda = 1, x variable

source('SimulationGamma_Masswechsel.R');
source('SimulationGamma.R');
source('ApproximationGamma2.R');
source('ExaktGamma.R');
source('ExaktGammaZ.R');

### Comparison A
# parameter of the poisson parameter: lambda = 1
# parameters of the gamma distribution: alpha = 2, beta = 1

lambda = 1;
alpha = 2;
beta = 1;
n = 100000;
accuracy = 10^(-10);

x = seq(0.1,20,by = 0.1)

# Compute the exact probability P(C>x) and the sensitivity of P(C>x) with
#   - the Monte Carlo Method ISS (Importance Sampling for Sensitivities)
#   - the Monte Carlo Method ISSEM (ISS with an exponential change of measure)
#   - a nearly exact calculation
#   - the saddlepoint approximation

sensitivity_sim = rep(0,length(x));
sensitivity_sim_M = rep(0,length(x));
sensitivity = rep(0,length(x));
sensitivity_appr = rep(0,length(x));
p = rep(0,length(x))

for (i in 1:length(x)){	
	sensitivity_sim[i] = SimulationGamma(x[i],alpha,beta,lambda,n=n);
	sensitivity_sim_M[i] = SimulationGamma_Masswechsel(x[i],alpha,beta,lambda,n=n);
	sensitivity[i] = ExaktGamma(x[i],alpha,beta,lambda,accuracy);
	sensitivity_appr[i] = ApproximationGamma2(x[i],alpha,beta,lambda);
	p[i] = ExaktGammaZ(x[i],alpha,beta,lambda,accuracy);
}


# values for upper axis
p_r = round(p, digits = 2);
p_s = signif(p, digits = 2);
p_axis_x =c(x[5],x[12],x[19],x[27],x[37],x[54],x[102],x[146],x[187]);
p_axis_p = c(p_r[5],p_r[12],p_r[19],p_r[27],p_r[37],p_r[54],p_r[102],p_s[146],p_s[187]);

# absolute and relative errors
absolute_error_sim = sensitivity_sim - sensitivity;
absolute_error_sim_M = sensitivity_sim_M - sensitivity;
absolute_error_appr = sensitivity_appr - sensitivity;

relative_error_sim = absolute_error_sim / sensitivity;
relative_error_sim_M = absolute_error_sim_M / sensitivity;
relative_error_appr = absolute_error_appr / sensitivity;


## Graphs

## Results: sensitivity & absolute error & relative error (detail)
# pdf file

pdf(file="A_Masswechsel.pdf",width=6, height=11.7);
par(mfrow=c(3,1),mar=c(5.7,4.1,6.1,2.1));

plot(x,sensitivity_sim_M,type="l",cex.axis=1.4,cex.lab=1.6,ylab="sensitvity",mgp=c(2.7,0.7,0),
     col="blue",lwd=0.5);
lines(x,sensitivity_appr,type="l",lty=2,col="blue",lwd=0.5);
lines(x,sensitivity,type="l",lty=3,col="blue",lwd=0.5);
legend("topright", c("Monte Carlo Method ISSEM","saddlepoint approximation","exact calculation"),
       cex=1.2,  lty=c(1,2,3),col="blue",lwd=0.5, bty="n");

axis(3,at=p_axis_x,labels=p_axis_p,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Sensitvity",font=2);


plot(x,absolute_error_sim_M,type="l",cex.axis=1.4,cex.lab=1.6,ylab="absolute error",
     mgp=c(2.7,0.7,0),col="blue",lwd=0.5);
lines(x,absolute_error_appr,type="l",lty=2,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
legend("topright", c("Monte Carlo Method ISSEM","saddlepoint approximation"), cex=1.2,  lty=1:2,
       col="blue",lwd=0.5, bty="n");

axis(3,at=p_axis_x,labels=p_axis_p,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Absolute Error of the Sensitvity",font=2);


plot(x,relative_error_sim_M,type="l",cex.axis=1.4,cex.lab=1.6,ylab="relative Error",
     mgp=c(2.7,0.7,0),col="blue",lwd=0.5,ylim=c(-0.02,0.02));
lines(x,relative_error_appr,type="l",lty=2,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
legend("topleft", c("Monte Carlo Method ISSEM","saddlepoint approximation"), cex=1.2,  lty=1:2,
       col="blue",lwd=0.5, bty="n");

axis(3,at=p_axis_x,labels=p_axis_p,cex.axis=1.4,mgp=c(1.7,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Relative Error of the Sensitvity",font=2);
dev.off()

# eps file
postscript(file="A_Masswechsel.eps", onefile=FALSE, horizontal=FALSE, width=6, height = 11.7)
par(mfrow=c(3,1),mar=c(5.7,4.1,6.1,2.1));

plot(x,sensitivity_sim_M,type="l",cex.axis=1.4,cex.lab=1.6,ylab="sensitvity",mgp=c(2.7,0.7,0),
     col="blue",lwd=0.5);
lines(x,sensitivity_appr,type="l",lty=2,col="blue",lwd=0.5);
lines(x,sensitivity,type="l",lty=3,col="blue",lwd=0.5);
legend("topright", c("Monte Carlo Method ISSEM","saddlepoint approximation","exact Calculation"),
       cex=1.2,  lty=c(1,2,3),col="blue",lwd=0.5, bty="n");

axis(3,at=p_axis_x,labels=p_axis_p,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Sensitvity",font=2);


plot(x,absolute_error_sim_M,type="l",cex.axis=1.4,cex.lab=1.6,ylab="absolute error",
     mgp=c(2.7,0.7,0),col="blue",lwd=0.5);
lines(x,absolute_error_appr,type="l",lty=2,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
legend("topright", c("Monte Carlo Method ISSEM","saddlepoint approximation"), cex=1.2,lty=1:2,
       col="blue",lwd=0.5, bty="n");

axis(3,at=p_axis_x,labels=p_axis_p,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Absolute Error of the Sensitvity",font=2);


plot(x,relative_error_sim_M,type="l",cex.axis=1.4,cex.lab=1.6,ylab="relative error",
     mgp=c(2.7,0.7,0),col="blue",lwd=0.5,ylim=c(-0.02,0.02));
lines(x,relative_error_appr,type="l",lty=2,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
legend("topleft", c("Monte Carlo Method ISSEM","saddlepoint approximation"), cex=1.2, lty=1:2,
       col="blue",lwd=0.5, bty="n");

axis(3,at=p_axis_x,labels=p_axis_p,cex.axis=1.4,mgp=c(1.7,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Relative Error of the Sensitvity",font=2);
dev.off()

### Results: sensitivity & absolute error & relative error

pdf(file="A_Masswechsel_Fehler.pdf",width=6, height=11.7);
par(mfrow=c(3,1),mar=c(5.7,4.1,6.1,2.1));

plot(x,sensitivity_sim_M,type="l",cex.axis=1.4,cex.lab=1.6,ylab="sensitvity",mgp=c(2.7,0.7,0),
     col="blue",lwd=0.5);
lines(x,sensitivity_appr,type="l",lty=2,col="blue",lwd=0.5);
lines(x,sensitivity,type="l",lty=3,col="blue",lwd=0.5);
legend("topright", c("Monte Carlo Method ISSEM","saddlepoint approximation","exact calculation"),
       cex=1.2,  lty=c(1,2,3),col="blue",lwd=0.5, bty="n");

axis(3,at=p_axis_x,labels=p_axis_p,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Sensitvity",font=2);


plot(x,absolute_error_appr,type="l",lty=2,cex.axis=1.4,cex.lab=1.6,ylab="absolute error",
     mgp=c(2.7,0.7,0),ylim=c(-0.06,0.01),col="blue",lwd=0.5);
lines(x,absolute_error_sim_M,type="l",lty=1,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
legend("bottomright", c("Monte Carlo Method ISSEM","saddlepoint approximation"), cex=1.2,lty=1:2,
       col="blue",lwd=0.5, bty="n");

axis(3,at=p_axis_x,labels=p_axis_p,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Absolute Error of the Sensitvity",font=2);


plot(x,relative_error_appr,type="l",lty=2,cex.axis=1.4,cex.lab=1.6,ylab="relative error",
     mgp=c(2.7,0.7,0),ylim=c(-0.15,0.03),col="blue",lwd=0.5);
lines(x,relative_error_sim_M,type="l",lty=1,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
legend("bottomright", c("Monte Carlo Method ISSEM","saddlepoint approximation"), cex=1.2,lty=1:2,
       col="blue",lwd=0.5, bty="n");

axis(3,at=p_axis_x,labels=p_axis_p,cex.axis=1.4,mgp=c(1.7,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Relative Error of hte Sensitvity",font=2);
dev.off()

# eps file
postscript(file="A_Masswechsel_Fehler.eps", onefile=FALSE, horizontal=FALSE, width=6, 
           height = 11.7)
par(mfrow=c(3,1),mar=c(5.7,4.1,6.1,2.1));

plot(x,sensitivity_sim_M,type="l",cex.axis=1.4,cex.lab=1.6,ylab="sensitvity",mgp=c(2.7,0.7,0),
     col="blue",lwd=0.5);
lines(x,sensitivity_appr,type="l",lty=2,col="blue",lwd=0.5);
lines(x,sensitivity,type="l",lty=3,col="blue",lwd=0.5);
legend("topright", c("Monte Carlo Method ISSEM","saddlepoint approximation","exact calculation"),
       cex=1.2,  lty=c(1,2,3),col="blue",lwd=0.5, bty="n");

axis(3,at=p_axis_x,labels=p_axis_p,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Sensitvity",font=2);


plot(x,absolute_error_appr,type="l",lty=2,cex.axis=1.4,cex.lab=1.6,ylab="absolute error",
     mgp=c(2.7,0.7,0),ylim=c(-0.06,0.01),col="blue",lwd=0.5);
lines(x,absolute_error_sim_M,type="l",lty=1,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
legend("bottomright", c("Monte Carlo Method ISSEM","saddlepoint approximation"), cex=1.2, lty=1:2,
       col="blue",lwd=0.5, bty="n");

axis(3,at=p_axis_x,labels=p_axis_p,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Absolute Error of the Sensitvity",font=2);


plot(x,relative_error_appr,type="l",lty=2,cex.axis=1.4,cex.lab=1.6,ylab="relative error",
     mgp=c(2.7,0.7,0),ylim=c(-0.15,0.03),col="blue",lwd=0.5);
lines(x,relative_error_sim_M,type="l",lty=1,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
legend("bottomright", c("Monte Carlo Method ISSEM","saddlepoint approximation"), cex=1.2, lty=1:2,
       col="blue",lwd=0.5, bty="n");

axis(3,at=p_axis_x,labels=p_axis_p,cex.axis=1.4,mgp=c(1.7,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Relative Error of the Sensitvity",font=2);
dev.off()

## Comparison ISS with ISSEM
par(mfrow=c(1,1),mar=c(5.7,4.1,6.1,2.1));
plot(x,relative_error_sim,type="l",lty=2,cex.axis=1.2,cex.lab=1.3,ylab="relative error",
     mgp=c(2.5,0.5,0),col="blue",lwd=0.5);
lines(x,relative_error_sim_M,type="l",lty=1,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
axis(3,at=p_axis_x,labels=p_axis_p,cex.axis=1.2,mgp=c(1.5,0.5,0));
mtext(side=3,line=1.6,cex=1.3,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.2, cex=1.8,"Relative Error of the Sensitvity",font=2);
legend("topleft", c("ISS","ISSEM"),
       cex=1.2,  lty=2:1,col="blue",lwd=0.5, bty="n");
dev.copy2pdf(file="A_Vergleich_mit_ohne_Masswechsel.pdf",width=8,height=5.5)
dev.copy2eps(file="A_Vergleich_mit_ohne_Masswechsel.eps",width=8,height=5.5)
dev.off()
