# author: Chantal Peeters
# date: 02.01.2013

### Computations to illustrate the comparison between the saddlepoint approximation
### and the Monte Carlo Methods ISS and ISSEM in the calculation of sensitivities of
### random sums.

### Random sum C = Compound Poisson Sum with Gamma distributed summands
### We calculate the sensitivity of P(C>x) with respect to parameter of the Poisson
### distribution (lambda).

### Comparison B: lambda variable, x = 10.25

source('SimulationGamma.R');
source('SimulationGamma_Masswechsel.R');
source('ApproximationGamma2.R');
source('ExaktGamma.R');
source('ExaktGammaZ.R');
 
### Comparison B: x = 10.25
# parameters of the gamma distribution: alpha = 2, beta = 1
x = 10.25;
alpha = 2;
beta = 1;
n = 100000;
genauigkeit = 10^(-20);

lambda = seq(0.08,14,by=0.08)

# Compute the exact probability P(C>x) and the sensitivity of P(C>x) with
#   - the Monte Carlo Method ISS (Importance Sampling for Sensitivities)
#   - the Monte Carlo Method ISSEM (ISS with an exponential change of measure)
#   - a nearly exact calculation
#   - the saddlepoint approximation

sensitivity_sim = rep(0,length(lambda))
sensitivity_sim_M = rep(0,length(lambda))
sensitivity = rep(0,length(lambda))
sensitivity_appr = rep(0,length(lambda))
p = rep(0,length(lambda))

for (i in 1:length(lambda)){
	sensitivity_sim[i] = SimulationGamma(x,alpha,beta,lambda[i],n=n);
	sensitivity_sim_M[i] = SimulationGamma_Masswechsel(x,alpha,beta,lambda[i],n=n);
	sensitivity[i] = ExaktGamma(x,alpha,beta,lambda[i],genauigkeit);
	sensitivity_appr[i] = ApproximationGamma2(x,alpha,beta,lambda[i]);
	p[i] = ExaktGammaZ(x,alpha,beta,lambda[i],genauigkeit);
}

# values for upper axis
p_r = round(p, digits = 2);
p_r_3 = round(p, digits = 3);
p_s = signif(p, digits = 2);
p_axis_lambda = c(lambda[5],lambda[13],lambda[31],lambda[42],lambda[51],
lambda[60],lambda[68],lambda[78],lambda[88],lambda[102],lambda[122],lambda[172]);
p_axis_p = c(p_r_3[5],p_r[13],p_r[31],p_r[42],p_r[51],p_r[60],p_r[68],p_r[78],
p_r[88],p_r[102],p_r[122],p_r[172]);

# absolute and relative errors
absoluter_fehler_sim = sensitivity_sim - sensitivity;
absoluter_fehler_sim_M = sensitivity_sim_M - sensitivity;
absoluter_fehler_appr = sensitivity_appr - sensitivity;

relativer_fehler_sim = absoluter_fehler_sim/sensitivity;
relativer_fehler_sim_M = absoluter_fehler_sim_M/sensitivity;
relativer_fehler_appr = absoluter_fehler_appr / sensitivity;


## Graphs

## Results: sensitivity & absolute error & relative error

# pdf file

pdf(file="B_Masswechsel.pdf",width=6, height=11.7);
par(mfrow=c(3,1),mar=c(5.7,4.1,6.1,2.1));

plot(lambda,sensitivity_sim_M,type="l",cex.axis=1.4,cex.lab=1.6,xlab=expression(lambda),ylab="Sensitivitt",mgp=c(2.7,0.7,0),col="blue",lwd=0.5);
lines(lambda,sensitivity_appr,type="l",lty=2,col="blue",lwd=0.5);
lines(lambda,sensitivity,type="l",lty=3,col="blue",lwd=0.5);
legend("topright", c("Monte Carlo Methode ISSEM","Sattelpunkt Approximation","exakte Berechnung"), cex=1.2,  lty=c(1,2,3),col="blue",lwd=0.5,bty="n");

axis(3,at=p_axis_lambda,labels=p_axis_p,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Sensitivitt",font=2);


plot(lambda,absoluter_fehler_sim_M,type="l",cex.axis=1.4,cex.lab=1.6,xlab=expression(lambda),ylab="absoluter Fehler",mgp=c(2.7,0.7,0),col="blue",lwd=0.5,ylim=c(-0.0026,0.0026));
lines(lambda,absoluter_fehler_appr,type="l",lty=2,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
legend("topleft", c("Monte Carlo Methode ISSEM","Sattelpunkt Approximation"), cex=1.2,  lty=1:2,col="blue",lwd=0.5, bty="n");

axis(3,at=p_axis_lambda,labels=p_axis_p,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Absoluter Fehler der Sensitivitt",font=2);


plot(lambda,relativer_fehler_sim_M,type="l",cex.axis=1.4,cex.lab=1.6,xlab=expression(lambda),ylab="relativer Fehler",mgp=c(2.7,0.7,0),col="blue",lwd=0.5,ylim=c(-0.23,0.23));
lines(lambda,relativer_fehler_appr,type="l",lty=2,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
legend("topleft", c("Monte Carlo Methode ISSEM","Sattelpunkt Approximation"), cex=1.2,  lty=1:2,col="blue",lwd=0.5, bty="n");

axis(3,at=p_axis_lambda,labels=p_axis_p,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Relativer Fehler der Sensitivitt",font=2);
dev.off();



# eps file
postscript(file="B_Masswechsel.eps", onefile=FALSE, horizontal=FALSE, width=6, height = 11.7)
par(mfrow=c(3,1),mar=c(5.7,4.1,6.1,2.1));

plot(lambda,sensitivity_sim_M,type="l",cex.axis=1.4,cex.lab=1.6,xlab=expression(lambda),ylab="Sensitivitt",mgp=c(2.7,0.7,0),col="blue",lwd=0.5);
lines(lambda,sensitivity_appr,type="l",lty=2,col="blue",lwd=0.5);
lines(lambda,sensitivity,type="l",lty=3,col="blue",lwd=0.5);
legend("topright", c("Monte Carlo Methode ISSEM","Sattelpunkt Approximation","exakte Berechnung"), cex=1.2,  lty=c(1,2,3),col="blue",lwd=0.5,bty="n");

axis(3,at=p_axis_lambda,labels=p_axis_p,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Sensitivitt",font=2);


plot(lambda,absoluter_fehler_sim_M,type="l",cex.axis=1.4,cex.lab=1.6,xlab=expression(lambda),ylab="absoluter Fehler",mgp=c(2.7,0.7,0),col="blue",lwd=0.5,ylim=c(-0.0026,0.0026));
lines(lambda,absoluter_fehler_appr,type="l",lty=2,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
legend("topleft", c("Monte Carlo Methode ISSEM","Sattelpunkt Approximation"), cex=1.2,  lty=1:2,col="blue",lwd=0.5, bty="n");

axis(3,at=p_axis_lambda,labels=p_axis_p,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Absoluter Fehler der Sensitivitt",font=2);


plot(lambda,relativer_fehler_sim_M,type="l",cex.axis=1.4,cex.lab=1.6,xlab=expression(lambda),ylab="relativer Fehler",mgp=c(2.7,0.7,0),col="blue",lwd=0.5,ylim=c(-0.23,0.23));
lines(lambda,relativer_fehler_appr,type="l",lty=2,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
legend("topleft", c("Monte Carlo Methode ISSEM","Sattelpunkt Approximation"), cex=1.2,  lty=1:2,col="blue",lwd=0.5, bty="n");

axis(3,at=p_axis_lambda,labels=p_axis_p,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Relativer Fehler der Sensitivitt",font=2);
dev.off();


## Comparison ISS with ISSEM
par(mfrow=c(1,1),mar=c(5.7,4.1,6.1,2.1));
plot(lambda,relativer_fehler_sim,type="l",lty=2,cex.axis=1.2,cex.lab=1.3,xlab=expression(lambda),ylab="relativer Fehler",mgp=c(2.5,0.5,0),xlim=c(0,10),ylim=c(-0.5,0.5),col="blue",lwd=0.5);
lines(lambda,relativer_fehler_sim_M,type="l",lty=1,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
axis(3,at=p_axis_lambda,labels=p_axis_p,cex.axis=1.2,mgp=c(1.5,0.5,0));
mtext(side=3,line=1.6,cex=1.3,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.2, cex=1.8,"Relativer Fehler der Sensitivitt",font=2);
legend("topleft", c("ohne exponentiellen Masswechsel","mit exponentiellen Masswechsel"), cex=1.2,  lty=2:1,col="blue",lwd=0.5, bty="n");
dev.copy2pdf(file="B_Vergleich_mit_ohne_Masswechsel.pdf",width=8,height=5.5)
dev.copy2eps(file="B_Vergleich_mit_ohne_Masswechsel.eps",width=8,height=5.5)
dev.off()
