# author: Chantal Peeters
# date: 02.01.2013

### Computations to illustrate the comparison between the saddlepoint approximation
### and the Monte Carlo Method ISS in the calculation of sensitivities of
### random sums.

### Random sum C = Compound Poisson Sum with Weibull distributed summands
### We calculate the sensitivity of P(C>x) with respect to the parameter of the Poisson
### distribution (lambda).

### Comparison A: lambda = 1, x variable

source('SimulationWeibull.R');
source('SimulationWeibullZ.R');
source('ApproximationWeibull.R');

 
### Comparison A: 
# parameter of the Poisson distribution: lambda = 1
# parameter of the Weibull distribution: c = 3

lambda = 1;
c = 3;
n = 100000;
n_exakt = 10000000;
genauigkeit = 10^(-12);

x = seq(0.02,6,by = 0.02)

# Compute the exact probability P(C>x) and the sensitivity of P(C>x) with
#   - the Monte Carlo Method ISS (Importance Sampling for Sensitivities)
#   - the Monte Carlo Method ISS with a very high simulation parameter
#   - the saddlepoint approximation

sensitivity_sim = rep(0,length(x));
sensitivity_sim_M = rep(0,length(x));
sensitivity = rep(0,length(x));
sensitivity_appr = rep(0,length(x));
p = rep(0,length(x))

for (i in 1:length(x)){
	print(i);
	sensitivity_sim[i] = SimulationWeibull(x[i],c,lambda,n=n);
	sensitivity[i] = SimulationWeibull(x[i],c,lambda,n=n_exakt)
	sensitivity_appr[i] = ApproximationWeibull(x[i],c,lambda,genauigkeit);
	p[i] = SimulationWeibullZ(x[i],c,lambda,n=n_exakt)

}

# values for upper axis
p_r = round(p, digits = 2);
p_s = signif(p, digits = 2);
p_axis_x =c(x[22],x[38],x[49],x[61],x[80],x[110],x[190],x[259],x[300]);
p_axis_p = c(p_r[22],p_r[38],p_r[49],p_r[61],p_r[80],p_r[110],p_r[190],p_s[259],p_s[300]);

# absolute and relative errors
absoluter_fehler_sim = sensitivity_sim - sensitivity;
absoluter_fehler_appr = sensitivity_appr - sensitivity;

relativer_fehler_sim = absoluter_fehler_sim/sensitivity;
relativer_fehler_appr = absoluter_fehler_appr / sensitivity;


## Graphs
## Results: sensitivity & absolute error & relative error
# pdf file

pdf(file="A.pdf",width=6, height=11.7);
par(mfrow=c(3,1),mar=c(5.7,4.1,6.1,2.1));

plot(x,sensitivity_sim,type="l",cex.axis=1.4,cex.lab=1.6,ylab="Sensitivitt",mgp=c(2.7,0.7,0),col="blue",lwd=0.5);
lines(x,sensitivity_appr,type="l",lty=2,col="blue",lwd=0.5);
lines(x,sensitivity,type="l",lty=3,col="blue",lwd=0.5);
legend("topright", c("Monte Carlo Methode ISS","Sattelpunkt Approximation","Referenz"), cex=1.2,  lty=c(1,2,3),col="blue",lwd=0.5, bty="n");

axis(3,at=p_axis_x,labels=p_axis_p,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Sensitivitt",font=2);


plot(x,absoluter_fehler_sim,type="l",cex.axis=1.4,cex.lab=1.6,ylab="absoluter Fehler",mgp=c(2.7,0.7,0),col="blue",lwd=0.5);
lines(x,absoluter_fehler_appr,type="l",lty=2,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
legend("topright", c("Monte Carlo Methode ISS","Sattelpunkt Approximation"), cex=1.2,  lty=1:2,col="blue",lwd=0.5, bty="n");

axis(3,at=p_axis_x,labels=p_axis_p,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Absoluter Fehler der Sensitivitt",font=2);


plot(x,relativer_fehler_sim,type="l",cex.axis=1.4,cex.lab=1.6,ylab="relativer Fehler",mgp=c(2.7,0.7,0),col="blue",lwd=0.5);
lines(x,relativer_fehler_appr,type="l",lty=2,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
legend("topleft", c("Monte Carlo Methode ISS","Sattelpunkt Approximation"), cex=1.2,  lty=1:2,col="blue",lwd=0.5, bty="n");

axis(3,at=p_axis_x,labels=p_axis_p,cex.axis=1.4,mgp=c(1.7,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Relativer Fehler der Sensitivitt",font=2);
dev.off()


# eps file
postscript(file="A.eps", onefile=FALSE, horizontal=FALSE, width=6, height = 11.7)
par(mfrow=c(3,1),mar=c(5.7,4.1,6.1,2.1));

plot(x,sensitivity_sim,type="l",cex.axis=1.4,cex.lab=1.6,ylab="Sensitivitt",mgp=c(2.7,0.7,0),col="blue",lwd=0.5);
lines(x,sensitivity_appr,type="l",lty=2,col="blue",lwd=0.5);
lines(x,sensitivity,type="l",lty=3,col="blue",lwd=0.5);
legend("topright", c("Monte Carlo Methode ISS","Sattelpunkt Approximation","Referenz"), cex=1.2,  lty=c(1,2,3),col="blue",lwd=0.5, bty="n");

axis(3,at=p_axis_x,labels=p_axis_p,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Sensitivitt",font=2);


plot(x,absoluter_fehler_sim,type="l",cex.axis=1.4,cex.lab=1.6,ylab="absoluter Fehler",mgp=c(2.7,0.7,0),col="blue",lwd=0.5);
lines(x,absoluter_fehler_appr,type="l",lty=2,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
legend("topright", c("Monte Carlo Methode ISS","Sattelpunkt Approximation"), cex=1.2,  lty=1:2,col="blue",lwd=0.5, bty="n");

axis(3,at=p_axis_x,labels=p_axis_p,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Absoluter Fehler der Sensitivitt",font=2);


plot(x,relativer_fehler_sim,type="l",cex.axis=1.4,cex.lab=1.6,ylab="relativer Fehler",mgp=c(2.7,0.7,0),col="blue",lwd=0.5);
lines(x,relativer_fehler_appr,type="l",lty=2,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
legend("topleft", c("Monte Carlo Methode ISS","Sattelpunkt Approximation"), cex=1.2,  lty=1:2,col="blue",lwd=0.5, bty="n");

axis(3,at=p_axis_x,labels=p_axis_p,cex.axis=1.4,mgp=c(1.7,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Relativer Fehler der Sensitivitt",font=2);
dev.off()