# author: Chantal Peeters
# date: 02.01.2013

### Computations to illustrate the comparison between the saddlepoint approximation
### and the Monte Carlo Method ISS in the calculation of sensitivities of
### random sums.

### Random sum C = Compound Poisson Sum with Weibull distributed summands
### We calculate the sensitivity of P(C>x) with respect to the parameter of the Poisson
### distribution (lambda).

### Comparison B: lambda = variable, x = 3.8
source('SimulationWeibull.R');
source('SimulationWeibullZ.R');
source('ApproximationWeibull2.R');

 
### Comparison B: x = 3.8
# parameter of the Weibull distribution: c = 3
x = 3.8;
beta = 3;
n = 100000;
n_exakt = 10000000;
genauigkeit = 10^(-12);

lambda = seq(0.15,15,by=0.075);

# Compute the exact probability P(C>x) and the sensitivity of P(C>x) with
#   - the Monte Carlo Method ISS (Importance Sampling for Sensitivities)
#   - the Monte Carlo Method ISS with a very high simulation parameter
#   - the saddlepoint approximation

sensitivity_sim = rep(0,length(lambda));
sensitivity = rep(0,length(lambda));
sensitivity_appr = rep(0,length(lambda));
p = rep(0,length(lambda));

for (i in 1:length(lambda)){
	print(i);
	sensitivity_sim[i] = SimulationWeibull(x,beta,lambda[i],n=n);
	sensitivity[i] = SimulationWeibull(x,beta,lambda[i],n=n_exakt);
	sensitivity_appr[i] = ApproximationWeibull2(x,beta,lambda[i],genauigkeit);
	p[i] = SimulationWeibullZ(x,beta,lambda[i],n=n_exakt);
}


# values for upper axis
p_r = round(p, digits = 2);
p_r_3 = round(p, digits = 3);
p_s = signif(p, digits = 2);
p_axis_lambda = c(lambda[1],lambda[6],lambda[13],lambda[28],lambda[37],lambda[44],
lambda[52],lambda[59],lambda[67],lambda[76],lambda[87],lambda[106],lambda[157],lambda[199]);
p_axis_p = c(p_s[1],p_r_3[6],p_r[13],p_r[28],p_r[37],p_r[44],p_r[52],p_r[59],p_r[67]
,p_r[76],p_r[87],p_r[106],p_r[157],p_r_3[199]);

# absolute and relative errors
absoluter_fehler_sim = sensitivity_sim - sensitivity;
absoluter_fehler_appr = sensitivity_appr - sensitivity;

relativer_fehler_sim = absoluter_fehler_sim/sensitivity;
relativer_fehler_appr = absoluter_fehler_appr / sensitivity;


## Graphs
## Results: sensitivity & absolute error & relative error
# pdf file

pdf(file="B.pdf",width=6, height=11.7);
par(mfrow=c(3,1),mar=c(5.7,4.1,6.1,2.1));

plot(lambda,sensitivity_sim,type="l",cex.axis=1.4,cex.lab=1.6,xlab=expression(lambda),ylab="Sensitivitt",mgp=c(2.7,0.7,0),col="blue",lwd=0.5);
lines(lambda,sensitivity_appr,type="l",lty=2,col="blue",lwd=0.5);
lines(lambda,sensitivity,type="l",lty=3,col="blue",lwd=0.5);
legend("topright", c("Monte Carlo Methode ISS","Sattelpunkt Approximation","Referenz"), cex=1.2,  lty=c(1,2,3),col="blue",lwd=0.5, bty="n");

axis(3,at=,p_axis_lambda,p_axis_p,labels=,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Sensitivitt",font=2);


plot(lambda,absoluter_fehler_sim,type="l",cex.axis=1.4,cex.lab=1.6,xlab=expression(lambda),ylab="absoluter Fehler",mgp=c(2.7,0.7,0),col="blue",lwd=0.5,ylim=c(-0.004,0.004));
lines(lambda,absoluter_fehler_appr,type="l",lty=2,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
legend("topright", c("Monte Carlo Methode ISS","Sattelpunkt Approximation"), cex=1.2,  lty=1:2,col="blue",lwd=0.5, bty="n");

axis(3,at=,p_axis_lambda,p_axis_p,labels=,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Absoluter Fehler der Sensitivitt",font=2);


plot(lambda,relativer_fehler_sim,type="l",cex.axis=1.4,cex.lab=1.6,xlab=expression(lambda),ylab="relativer Fehler",mgp=c(2.7,0.7,0),col="blue",lwd=0.5,ylim=c(-1.4,1.4));
lines(lambda,relativer_fehler_appr,type="l",lty=2,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
legend("bottomleft", c("Monte Carlo Methode ISS","Sattelpunkt Approximation"), cex=1.2,  lty=1:2,col="blue",lwd=0.5, bty="n");

axis(3,at=,p_axis_lambda,p_axis_p,labels=,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Relativer Fehler der Sensitivitt",font=2);
dev.off();

# eps file
postscript(file="B.eps", onefile=FALSE, horizontal=FALSE, width=6, height = 11.7)
par(mfrow=c(3,1),mar=c(5.7,4.1,6.1,2.1));

plot(lambda,sensitivity_sim,type="l",cex.axis=1.4,cex.lab=1.6,xlab=expression(lambda),ylab="Sensitivitt",mgp=c(2.7,0.7,0),col="blue",lwd=0.5);
lines(lambda,sensitivity_appr,type="l",lty=2,col="blue",lwd=0.5);
lines(lambda,sensitivity,type="l",lty=3,col="blue",lwd=0.5);
legend("topright", c("Monte Carlo Methode ISS","Sattelpunkt Approximation","Referenz"), cex=1.2,  lty=c(1,2,3),col="blue",lwd=0.5, bty="n");

axis(3,at=,p_axis_lambda,p_axis_p,labels=,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Sensitivitt",font=2);


plot(lambda,absoluter_fehler_sim,type="l",cex.axis=1.4,cex.lab=1.6,xlab=expression(lambda),ylab="absoluter Fehler",mgp=c(2.7,0.7,0),col="blue",lwd=0.5,ylim=c(-0.004,0.004));
lines(lambda,absoluter_fehler_appr,type="l",lty=2,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
legend("topright", c("Monte Carlo Methode ISS","Sattelpunkt Approximation"), cex=1.2,  lty=1:2,col="blue",lwd=0.5, bty="n");

axis(3,at=,p_axis_lambda,p_axis_p,labels=,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Absoluter Fehler der Sensitivitt",font=2);


plot(lambda,relativer_fehler_sim,type="l",cex.axis=1.4,cex.lab=1.6,xlab=expression(lambda),ylab="relativer Fehler",mgp=c(2.7,0.7,0),col="blue",lwd=0.5,ylim=c(-1.4,1.4));
lines(lambda,relativer_fehler_appr,type="l",lty=2,col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
legend("bottomleft", c("Monte Carlo Methode ISS","Sattelpunkt Approximation"), cex=1.2,  lty=1:2,col="blue",lwd=0.5, bty="n");

axis(3,at=,p_axis_lambda,p_axis_p,labels=,cex.axis=1.4,mgp=c(1.5,0.7,0));
mtext(side=3,line=1.8,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.9, cex=1.2,"Relativer Fehler der Sensitivitt",font=2);
dev.off();